//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Top file of the SOPC
//
module sopc (
  clk_i, clk_p_i, clk_n_i, btn_i, sw_i,
  // UART interface
  sin_i,
  sout_o,
  // Test I/O
  led_o
  
  );
  input clk_i;          // Input clock 33 MHz
  input clk_p_i;        // Positive differential clock
  input clk_n_i;        // Negative differential clock
  input [3:0] btn_i;    // Input buttons
  input [3:0] sw_i;     // Input dip switches
  // UART interface
  input sin_i;          // Serial input
  output sout_o;        // Serial output
  // Test I/O
  output [3:0] led_o;    // Output LED signals
  
  // Internal registers and wires

  
  reg init_done = 1'b0;
  reg [15:0] init_counter = 16'h0;
  wire rst_w = (~init_done) ? 1'b0 : ~btn_i[0];

  wire [31:0] wb_adr_w;
  wire [31:0] wb_data_w;
  wire wb_we_w;
  wire [3:0] wb_sel_w;
  wire wb_stb_w;
  wire wb_ack_w;

  wire [31:0] uart_adr_w;
  wire [31:0] uart_data_w;
  wire uart_we_w;
  wire [3:0] uart_sel_w;
  wire uart_stb_w;
  wire uart_cyci_w;
  wire uart_ack_w;
  wire uart_cyco_w;

  wire [31:0] fft_data_w;
  wire [31:0] fft_adr_w;
  wire fft_we_w;
  wire [3:0] fft_sel_w;
  wire fft_stb_w;
  wire fft_cyci_w;
  wire fft_ack_w;
  wire fft_cyco_w;  
  
  wire [13:0] re_data_w;
  wire [13:0] re2_data_w;
  wire [13:0] im_data_w;
  wire [13:0] im2_data_w;
  wire conv_start_w;
  wire dvalid_w;
  wire data_clk_w;
  wire fft_clk_w;
  wire [5:0] idx_w;
  wire [3:0] test1_w;
  wire [3:0] test2_w;
  wire [3:0] uart_test1_w;
  wire [3:0] wbhost_test1_w;
  wire conv_rdy_w;
  wire [17:0] re_w;
  wire [17:0] im_w;
  wire [6:0] adr_w;
  wire rd_w;
  

  assign led_o = (sw_i == 4'h0) ? test1_w :
                 (sw_i == 4'h1) ? test2_w :
                 (sw_i == 4'h2) ? uart_test1_w :
                 (sw_i == 4'h3) ? wbhost_test1_w :
                 (sw_i == 4'h4) ? 4'b1010 : 4'h0; 
/*
//------------Test assignments-------------------------
  assign fft_clk_w = clk_p_i;
  //----------End of test assignments--------------------
*/   
  // IP modules

    clk_wiz_v3_2_0 diff_clk
     (// Clock in ports
      .CLK_IN1_P(clk_p_i),    // IN
      .CLK_IN1_N(clk_n_i),    // IN
      // Clock out ports
      .CLK_OUT1(fft_clk_w));    // OUT
  
  
  // End of IP modules
/*    
  //-----------------------Chipscope logic-----------------------------
    wire [35:0] scope_cntr_w;
    
    chipscope_icon_v1_06_a_0 scope_cntr1 (
      .CONTROL0(scope_cntr_w) // INOUT BUS [35:0]
  );
  
    chipscope_ila_v1_05_a_0 scope_ila1 (
      .CONTROL(scope_cntr_w), // INOUT BUS [35:0]
      .CLK(fft_clk_w), // IN
      .TRIG0(test1_w), // IN BUS [3:0]
      .TRIG1({12'h0,2'b00,idx_w}), // IN BUS [19:0]
      .TRIG2({6'h0,re_data_w}), // IN BUS [19:0]
      .TRIG3({6'h0,re2_data_w}), // IN BUS [19:0]
      .TRIG4({6'h0,im_data_w}) // IN BUS [19:0]
  );
  //----------------------End of chipscope logic-----------------------  
*/  

  // Modules instantiation
  fft_top fft_sopc (
    .clk_i              (clk_i),
    .rst_i              (rst_w),
    .data_clk_i         (clk_i),        // data_clk_w 
    .re_data_i          (re_data_w),
    .re2_data_i         (re2_data_w),
    .im_data_i          (im_data_w), 
    .im2_data_i         (im2_data_w),
    .conv_start_i       (conv_start_w),
    .dvalid_i           (dvalid_w),
    .rd_i               (rd_w),
    .re_data_o          (re_w), 
    .im_data_o          (im_w),
    .conv_rdy_o         (conv_rdy_w), 
    .adr_o              (adr_w),
    .idx_o              (idx_w),
    .test1_o            (test1_w),
    .test2_o            (test2_w)
  );

  wb_fft wb_fft_sopc (
    .clk_i            (clk_i),
    .fft_clk_i        (fft_clk_w),
    .rst_i            (rst_w),
    .wb_adr_i         (wb_adr_w), 
    .wb_data_i        (wb_data_w), 
    .wb_we_i          (wb_we_w), 
    .wb_sel_i         (wb_sel_w), 
    .wb_stb_i         (wb_stb_w), 
    .wb_ack_i         (wb_ack_w),
    .wb_cyc_i         (fft_cyco_w),    
    .wb_data_o        (fft_data_w),
    .wb_adr_o         (fft_adr_w),
    .wb_we_o          (fft_we_w),
    .wb_sel_o         (fft_sel_w),
    .wb_stb_o         (fft_stb_w),
    .wb_cyc_o         (fft_cyci_w),
    .wb_ack_o         (fft_ack_w),
    .idx_i            (idx_w),
    .conv_rdy_i       (conv_rdy_w),
    .re_i             (re_w),
    .im_i             (im_w),
    .adr_i            (adr_w),
    .re_data_o        (re_data_w),
    .re2_data_o       (re2_data_w),
    .im_data_o        (im_data_w),
    .im2_data_o       (im2_data_w),
    .conv_start_o     (conv_start_w),
    .dvalid_o         (dvalid_w),
    .data_clk_o       (data_clk_w),
    .rd_o             (rd_w)
  );
  
  ifft_top ifft_sopc(
    .clk_i              (),
    .rst_i              ()
  );
  
  uart_wb uart_sopc (
  .clk_i                      (clk_i), 
  .rst_i                      (rst_w), 
  .sin_i                      (sin_i),
  .sout_o                     (sout_o), 
  .uart_led_o                 (uart_test1_w),
  .wb_data_i                  (wb_data_w), 
  .wb_adr_i                   (wb_adr_w), 
  .wb_we_i                    (wb_we_w), 
  .wb_sel_i                   (wb_sel_w), 
  .wb_stb_i                   (wb_stb_w), 
  .wb_ack_i                   (wb_ack_w), 
  .wb_cyc_i                   (uart_cyco_w),
  .wb_data_o                  (uart_data_w), 
  .wb_adr_o                   (uart_adr_w), 
  .wb_we_o                    (uart_we_w), 
  .wb_sel_o                   (uart_sel_w), 
  .wb_stb_o                   (uart_stb_w), 
  .wb_ack_o                   (uart_ack_w), 
  .wb_cyc_o                   (uart_cyci_w)
);

wb_host wb_sopc (
  .clk_i                      (clk_i),
  .rst_i                      (rst_w),
    // WB signals common to all devices
  .wb_adr_o                   (wb_adr_w), 
  .wb_data_o                  (wb_data_w), 
  .wb_we_o                    (wb_we_w), 
  .wb_sel_o                   (wb_sel_w), 
  .wb_stb_o                   (wb_stb_w), 
  .wb_ack_o                   (wb_ack_w),
  // Interface signals with UART controller
  .uart_adr_i                 (uart_adr_w), 
  .uart_data_i                (uart_data_w), 
  .uart_we_i                  (uart_we_w), 
  .uart_sel_i                 (uart_sel_w), 
  .uart_stb_i                 (uart_stb_w), 
  .uart_cyc_i                 (uart_cyci_w), 
  .uart_ack_i                 (uart_ack_w),
  .uart_cyc_o                 (uart_cyco_w),
  // Interface signals with FFT WB module
  .fft_adr_i                  (fft_adr_w),
  .fft_data_i                 (fft_data_w),
  .fft_we_i                   (fft_we_w),
  .fft_sel_i                  (fft_sel_w),
  .fft_stb_i                  (fft_stb_w),
  .fft_cyc_i                  (fft_cyci_w),
  .fft_ack_i                  (fft_ack_w),
  .fft_cyc_o                  (fft_cyco_w),
  // Test I/O
  .test1_o                    (wbhost_test1_w)
);

  always @ (posedge clk_i)
    begin
      if  (init_counter != 16'h0FFF)
        begin
          init_counter <= init_counter + 1'b1;
          init_done <= 1'b0;
        end  
      else
        init_done <= 1'b1;
    end
  
endmodule